import _ from 'lodash';
import Boom from 'boom';
import Joi from 'joi';
import { getIsValidUser } from '../../../lib/get_is_valid_user';
import { getCalculateExpires } from '../../../lib/get_calculate_expires';
import { wrapError } from '../../../lib/errors';

export function initAuthenticateApi(server) {
  const isValidUser = getIsValidUser(server);
  const calculateExpires = getCalculateExpires(server);

  server.route({
    method: 'POST',
    path: '/api/security/v1/login',
    handler(request, reply) {
      const { username, password } = request.payload;
      return isValidUser(request, username, password).then((response) => {
        // Initialize the session
        request.cookieAuth.set({
          username,
          password,
          expires: calculateExpires()
        });

        return reply(response);
      }, (error) => {
        request.cookieAuth.clear();
        return reply(Boom.unauthorized(error));
      });
    },
    config: {
      auth: false,
      validate: {
        payload: {
          username: Joi.string().required(),
          password: Joi.string().required()
        }
      }
    }
  });

  server.route({
    method: 'POST',
    path: '/api/security/v1/logout',
    handler(request, reply) {
      request.cookieAuth.clear();
      return reply().code(204);
    },
    config: {
      auth: false,
    }
  });

  server.route({
    method: 'GET',
    path: '/api/security/v1/me',
    handler(request, reply) {
      server.plugins.security.getUser(request).then(reply, _.flow(wrapError, reply));
    }
  });
}